/******************************************************************************
 * 
 * Filename: config.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Machine dependant macros.
 *
 * (c) Copyright R.H.Thornburrow, 1998-2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#ifndef __config_h

#define __config_h

/******************************************************************************
 * User-Defineable Properties
 * ==========================
 * These defines control various aspects of the VM.  Feel free to change
 * anything in here by commenting it in or out.
 *****************************************************************************/

/* Define the JDK version to be used */
#ifndef JDK_VERSION

#define JDK_VERSION 118
/*#define JDK_VERSION 140*/

#endif /* JDK_VERSION */


/* Use some memory to cache the contents of zip files */
#define USE_ZIP_HASHTABLE 1

/* Don't use any zip files at all - removes the zip code */
/*#define USES_UNZIPPED_CLASSES_ONLY 1*/

/******************************************************************************
 * End of User-Defineable Properties
 * =================================
 * Do not change anything after this point unless you really know what you are
 * doing - things will break.
 *****************************************************************************/


/******************************************************************************
 * Global Properties
 *****************************************************************************/

#define JAVAVM_VERSION "1.1.0"

#ifdef _DEBUG
#define JAVAVM_RELEASE "DBG"
#else
#define JAVAVM_RELEASE "REL"
#endif

#if JDK_VERSION < 110
#error "JDK_VERSION must NOT be less than 1.1.0"
#elif JDK_VERSION < 120
#define JAVA_SPEC_VERSION  "1.1"
#define JAVA_CLASS_VERSION "45.3"
#define MAX_CLASS_VERSION  0x002e0000 /* 46.0 */
#elif JDK_VERSION < 140
#error "JDK 1.2 and 1.3 not tested yet!"
#elif JDK_VERSION < 150
#define JAVA_SPEC_VERSION  "1.4"
#define JAVA_CLASS_VERSION "48.0"
#define MAX_CLASS_VERSION  0x00310000 /* 49.0 */
#else
#error "JDK_VERSION must NOT be greater than 1.4.0"
#endif /* JDK_VERSION */


/******************************************************************************
 * System Dependant Properties
 *****************************************************************************/

 /******************************************************************************
 * Windows NT/2k/XP/9x Win32 (MSVC)
 */
#ifdef WIN32

#define JAVAVM_PLATFORM "WIN32"
#define PLATFORM_WIN32 1

#define CONFIG_LITTLE_ENDIAN 1
#define SUPPORTS_INT64 1

#define INT32_TYPE long
#define INT64_TYPE __int64
#define UINT32_TYPE unsigned long

#define FILE_PTR FILE*

#define USES_ENVIRONMENT_VARIABLES 1
#define USES_RELATIVE_FILENAMES 1
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".class"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
#define SUPPORTS_SIGNALS 1
/* #define USES_UNIXFILESYSTEM 0 */

/*#define LINE_SEPARATOR "\n\r"*/
#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "\\"
#define PATH_SEPARATOR ";"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "x86"
#define OS_NAME "Windows"

#define strdup _strdup
#define inline __inline

#endif /* WIN32 */


/******************************************************************************
 * Linux-i386 Intel-386 (GCC)
 */
#if defined(__linux__) && defined(__i386__) && defined(__GNUC__)

#define JAVAVM_PLATFORM "Linux-i386"
#define PLATFORM_LINUX_I386

#define CONFIG_LITTLE_ENDIAN 1
#define SUPPORTS_INT64 1

#define INT32_TYPE long
#define INT64_TYPE long long int
#define UINT32_TYPE unsigned long

#define FILE_PTR FILE*

#define USES_ENVIRONMENT_VARIABLES 1
#define USES_RELATIVE_FILENAMES 1
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".class"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
#define SUPPORTS_SIGNALS 1
#define USES_UNIXFILESYSTEM 1

#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "/"
#define PATH_SEPARATOR ":"
#define LIBRARY_EXTENSION "so"
#define OS_ARCH "x86"
#define OS_NAME "Linux"

#endif /* LINUX_I386 */


/******************************************************************************
 * Cygwin32 Intel-386 (GCC/ECGS)
 */
#if defined(__CYGWIN32__) && defined(__i386__) && defined(__GNUC__)

#define JAVAVM_PLATFORM "CYGWIN32"
#define PLATFORM_CYGWIN32

#define CONFIG_LITTLE_ENDIAN 1
#define SUPPORTS_INT64 1

#define INT32_TYPE long
#define INT64_TYPE long long int
#define UINT32_TYPE unsigned long

#define FILE_PTR FILE*

#define USES_ENVIRONMENT_VARIABLES 1
#define USES_RELATIVE_FILENAMES 1
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".class"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
#define SUPPORTS_SIGNALS 1
#define USES_UNIXFILESYSTEM 1

#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "/"
#define PATH_SEPARATOR ":"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "x86"
#define OS_NAME "Cygwin"

#endif /* LINUX_386 */


/******************************************************************************
 * Psion Series 3 Platform SIBO/EPOC16 (BCC)
 */
#if defined(__EPOC__) && defined(__BCC__)

#define JAVAVM_PLATFORM "EPOC16"
#define PLATFORM_EPOC16

#define CONFIG_LITTLE_ENDIAN 1
/* #define SUPPORTS_INT64 0 */

#define INT32_TYPE long
#define UINT32_TYPE unsigned long

#define DO_NOT_USE_LIBC 1

#define FILE_PTR unsigned short

/* #define USES_ENVIRONMENT_VARIABLES 0 */
#define USES_BACKSLASHED_FILENAMES 1
#define USES_EIGHTDOTTHREE_FILENAMES 1
#define USES_ABSOLUTE_FILENAMES 1
#define ABSOLUTE_FILENAME_ROOT "M:\\CLASSES"
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".cla"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
/* #define USES_UNIXFILESYSTEM 0 */

#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "\\"
#define PATH_SEPARATOR ";"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "x86"
#define OS_NAME "SIBO"

#define inline 

#endif /* EPOC16 */


/******************************************************************************
 * MSDOS Platform Intel 8086 (BCC)
 */
#if defined(__MSDOS__) && defined(__BCC__)

#define JAVAVM_PLATFORM "MSDOS"
#define PLATFORM_MSDOS

#define CONFIG_LITTLE_ENDIAN 1
/* #define SUPPORTS_INT64 0 */

#define INT32_TYPE long
#define UINT32_TYPE unsigned long

#define DO_NOT_USE_LIBC 1

#define FILE_PTR FILE*

/* #define USES_ENVIRONMENT_VARIABLES 0 */
#define USES_BACKSLASHED_FILENAMES 1
#define USES_EIGHTDOTTHREE_FILENAMES 1
#define USES_ABSOLUTE_FILENAMES 1
#define ABSOLUTE_FILENAME_ROOT "CLASSES"
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".cla"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
/* #define USES_UNIXFILESYSTEM 0 */

#define LINE_SEPARATOR "\n\r"
#define FILE_SEPARATOR "\\"
#define PATH_SEPARATOR ";"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "x86"
#define OS_NAME "DOS"

#define inline 

#ifdef _DEBUG
#define malloc debug_malloc
#endif /* _DEBUG */

#endif /* MSDOS */


/******************************************************************************
 * Psion Series 5 Emulator EPOC32/WINS (MSVC)
 */
#if defined(__SYMBIAN32__) && defined(__WINS__)

#define JAVAVM_PLATFORM "WINS"
#define PLATFORM_EPOC32_WINS

#define CONFIG_LITTLE_ENDIAN 1
#define SUPPORTS_INT64 1

#define INT32_TYPE long
#define INT64_TYPE __int64
#define UINT32_TYPE unsigned long

#define FILE_PTR FILE*

/* #define USES_ENVIRONMENT_VARIABLES 0 */
#define USES_BACKSLASHED_FILENAMES 1
#define USES_ABSOLUTE_FILENAMES 1
#define ABSOLUTE_FILENAME_ROOT "C:\\classes\\"
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".class"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
/* #define USES_UNIXFILESYSTEM 0 */

#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "\\"
#define PATH_SEPARATOR ";"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "x86"
#define OS_NAME "WINS"

#define inline __inline

#pragma warning( disable : 4100 4101 4127 4189 4244 4701 4706 )  

#endif /* EPOC32_WINS */


/******************************************************************************
 * Psion Series 5 EPOC32/MARM (GCC)
 */
#if defined(__EPOC32__) && defined(__MARM__) && defined(__GCC32__)

#define JAVAVM_PLATFORM "MARM"
#define PLATFORM_EPOC32_MARM

#define CONFIG_LITTLE_ENDIAN 1
#define SUPPORTS_INT64 1

#define INT32_TYPE long
#define INT64_TYPE long long int
#define UINT32_TYPE unsigned long

#define FILE_PTR FILE*

/* #define USES_ENVIRONMENT_VARIABLES 0 */
#define USES_BACKSLASHED_FILENAMES 1
#define USES_ABSOLUTE_FILENAMES 1
#define ABSOLUTE_FILENAME_ROOT "C:\\classes\\"
#define USES_FILE_EXTENSIONS 1
#define CLASS_FILE_EXTN ".class"
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
/* #define USES_UNIXFILESYSTEM 0 */

#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "\\"
#define PATH_SEPARATOR ";"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "ARM"
#define OS_NAME "MARM"

#endif /* EPOC32_MARM */


/******************************************************************************
 * Acorn Archimedes RISC OS (GCC)
 */
#if defined(__arm__) && defined(__riscos__) && defined(__GNUC__)

#define JAVAVM_PLATFORM "RISCOS"
#define PLATFORM_ARM_RISCOS

#define CONFIG_LITTLE_ENDIAN 1
#define SUPPORTS_INT64 1

#define INT32_TYPE long
#define INT64_TYPE long long int
#define UINT32_TYPE unsigned long

#define FILE_PTR FILE*

#define USES_ENVIRONMENT_VARIABLES 1
#define USES_ABSOLUTE_FILENAMES 1
#define ABSOLUTE_FILENAME_ROOT "/Classes/"
#define USES_NONDOLLAR_FILENAMES 1
/* #define USES_FILE_EXTENSIONS 0 */
#define USES_ZIP_FILE_EXTENSIONS 1
#define ZIP_CLASS_FILE_EXTN ".class"
#define USES_UNIXFILESYSTEM 1

#define LINE_SEPARATOR "\n"
#define FILE_SEPARATOR "/"
#define PATH_SEPARATOR ":"
#define LIBRARY_EXTENSION "dll"
#define OS_ARCH "ARM"
#define OS_NAME "RISCOS"

#endif /* ARM_RISCOS */


/*****************************************************************************/

/* Sanity checks on config options */

#ifndef JAVAVM_PLATFORM
#error "Unable to build for this platform"
#endif /* JAVAVM_PLATFORM */

#if !defined(CONFIG_LITTLE_ENDIAN) && !defined(CONFIG_BIG_ENDIAN)
#error "Must define either CONFIG_LITTLE_ENDIAN or CONFIG_BIG_ENDIAN"
#endif

#if !defined(INT32_TYPE) || !defined(UINT32_TYPE)
#error "32-bit integer types not fully defined"
#endif

#ifdef SUPPORTS_INT64
#if !defined(INT64_TYPE)
#error "64-bit integer type selected, but not defined"
#endif
#endif /* SUPPORTS_INT64 */

#if USES_FILE_EXTENSIONS && !defined(CLASS_FILE_EXTN)
#error "Must define CLASS_FILE_EXTN if USES_FILE_EXTENSIONS is 1"
#endif /* ordinary file extensions */

#if USES_ZIP_FILE_EXTENSIONS && !defined(ZIP_CLASS_FILE_EXTN)
#error "Must define ZIP_CLASS_FILE_EXTN if USES_ZIP_FILE_EXTENSIONS is 1"
#endif /* ordinary file extensions */


/*****************************************************************************/

#ifdef DO_NOT_USE_LIBC
/* for testing purposes only */
#define memset javavm_memset
#define memcpy javavm_memcpy
#define strncpy javavm_strncpy
#define strcpy javavm_strcpy
#define strncmp javavm_strncmp
#define strcmp javavm_strcmp
#define strlen javavm_strlen
#define strdup javavm_strdup
#endif /* DO_NOT_USE_LIBC */

#endif /* __config_h */
