/******************************************************************************
 *
 * Filename: disasm.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Class file disassembler interface.
 *
 * (c) Copyright R.H.Thornburrow, 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#ifndef _disasm_h

#define _disasm_h


#include "types.h"
#include "classfile.h"


#ifdef __cplusplus
extern "C" {
#endif

extern int sprintUtf8( char* buf, const struct _CONSTANT_Utf8_info* utf8 );

#ifdef _DEBUG
extern void disassemble( ClassFile *cf, int level );

extern void writeUTF8( u1* bytes, u2 count );
#endif /* _DEBUG */

extern int checkSimpleName( ClassFile* cf, u2 index, const char* compare );

#ifdef _DEBUG
extern int sprintTypeName( char* buf, u1 t );
extern int sprintDescriptor( char* buf, ClassFile* cf, u2 index );
extern int sprintMethodParameters( char* buf, ClassFile* cf, u2 index );
extern int sprintMethodReturn( char* buf, ClassFile* cf, u2 index );
#endif /* _DEBUG */

extern int sprintRawSimpleName( char* buf, ClassFile* cf, u2 index );
extern int sprintSimpleName( char* buf, ClassFile* cf, u2 index );
extern int sprintClassName( char* buf, ClassFile *cf, u2 index );

#ifdef _DEBUG
extern int sprintNameAndType( char* buf, ClassFile* cf, u2 index );
#endif /* _DEBUG */

extern int sprintFieldRef( char* buf, ClassFile *cf, u2 index );
extern int sprintMethodRef( char* buf, ClassFile *cf, u2 index );
extern int sprintInterfaceMethodRef( char* buf, ClassFile *cf, u2 index );
extern int sprintMethodInfo( char* buf, method_info* mi );

#ifdef _DEBUG
extern int sprintInstruction( char* buf, ClassFile* cf, u1* code, u4* n , int comments );
#endif /* _DEBUG */

#ifdef __cplusplus
}
#endif


#endif
