/******************************************************************************
 *
 * Filename: instructions.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Virtual instruction definitions.
 *
 * (c) Copyright R.H.Thornburrow, 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include "config.h"
#include "instructions.h"

#ifndef NULL
#define NULL (void*) 0
#endif

/* format encoding:
 *
 * B = byte  (1 byte)
 * S = short (2 bytes)
 * I = int   (4 bytes)
 * J = long  (8 bytes)
 *
 * c = Class     (2 bytes)
 * f = FieldRef  (2 bytes)
 * i = Literal   (1 byte)
 * l = Literal   (2 bytes)
 * m = MethodRef (2 bytes)
 * t = Type      (1 byte)
 * v = InterfaceMethodRef (2 bytes)
 */

InstructionTable instructionTable[] =
{
	/* opcode, size, incr, name, format, count  */
	INSTRUCTION_DEF( 0x00, 1, 1, "nop", NULL ),
	INSTRUCTION_DEF( 0x01, 1, 1, "aconst_null", NULL ),
	INSTRUCTION_DEF( 0x02, 1, 1, "iconst_m1", NULL ),
	INSTRUCTION_DEF( 0x03, 1, 1, "iconst_0", NULL ),
	INSTRUCTION_DEF( 0x04, 1, 1, "iconst_1", NULL ),
	INSTRUCTION_DEF( 0x05, 1, 1, "iconst_2", NULL ),
	INSTRUCTION_DEF( 0x06, 1, 1, "iconst_3", NULL ),
	INSTRUCTION_DEF( 0x07, 1, 1, "iconst_4", NULL ),
	INSTRUCTION_DEF( 0x08, 1, 1, "iconst_5", NULL ),
	INSTRUCTION_DEF( 0x09, 1, 1, "lconst_0", NULL ),
	INSTRUCTION_DEF( 0x0a, 1, 1, "lconst_1", NULL ),
	INSTRUCTION_DEF( 0x0b, 1, 1, "fconst_0", NULL ),
	INSTRUCTION_DEF( 0x0c, 1, 1, "fconst_1", NULL ),
	INSTRUCTION_DEF( 0x0d, 1, 1, "fconst_2", NULL ),
	INSTRUCTION_DEF( 0x0e, 1, 1, "dconst_0", NULL ),
	INSTRUCTION_DEF( 0x0f, 1, 1, "dconst_1", NULL ),
	INSTRUCTION_DEF( 0x10, 2, 2, "bipush", "B" ),
	INSTRUCTION_DEF( 0x11, 3, 3, "sipush", "S" ),
	INSTRUCTION_DEF( 0x12, 2, 2, "ldc",    "i" ),
	INSTRUCTION_DEF( 0x13, 3, 3, "ldc_w",  "l" ),
	INSTRUCTION_DEF( 0x14, 3, 3, "ldc2_w", "l" ),
	INSTRUCTION_DEF( 0x15, 2, 2, "iload",  "B" ),
	INSTRUCTION_DEF( 0x16, 2, 2, "lload",  "B" ),
	INSTRUCTION_DEF( 0x17, 2, 2, "fload",  "B" ),
	INSTRUCTION_DEF( 0x18, 2, 2, "dload",  "B" ),
	INSTRUCTION_DEF( 0x19, 2, 2, "aload",  "B" ),
	INSTRUCTION_DEF( 0x1a, 1, 1, "iload_0", NULL ),
	INSTRUCTION_DEF( 0x1b, 1, 1, "iload_1", NULL ),
	INSTRUCTION_DEF( 0x1c, 1, 1, "iload_2", NULL ),
	INSTRUCTION_DEF( 0x1d, 1, 1, "iload_3", NULL ),
	INSTRUCTION_DEF( 0x1e, 1, 1, "lload_0", NULL ),
	INSTRUCTION_DEF( 0x1f, 1, 1, "lload_1", NULL ),
	INSTRUCTION_DEF( 0x20, 1, 1, "lload_2", NULL ),
	INSTRUCTION_DEF( 0x21, 1, 1, "lload_3", NULL ),
	INSTRUCTION_DEF( 0x22, 1, 1, "fload_0", NULL ),
	INSTRUCTION_DEF( 0x23, 1, 1, "fload_1", NULL ),
	INSTRUCTION_DEF( 0x24, 1, 1, "fload_2", NULL ),
	INSTRUCTION_DEF( 0x25, 1, 1, "fload_3", NULL ),
	INSTRUCTION_DEF( 0x26, 1, 1, "dload_0", NULL ),
	INSTRUCTION_DEF( 0x27, 1, 1, "dload_1", NULL ),
	INSTRUCTION_DEF( 0x28, 1, 1, "dload_2", NULL ),
	INSTRUCTION_DEF( 0x29, 1, 1, "dload_3", NULL ),
	INSTRUCTION_DEF( 0x2a, 1, 1, "aload_0", NULL ),
	INSTRUCTION_DEF( 0x2b, 1, 1, "aload_1", NULL ),
	INSTRUCTION_DEF( 0x2c, 1, 1, "aload_2", NULL ),
	INSTRUCTION_DEF( 0x2d, 1, 1, "aload_3", NULL ),
	INSTRUCTION_DEF( 0x2e, 1, 1, "iaload", NULL ),
	INSTRUCTION_DEF( 0x2f, 1, 1, "laload", NULL ),
	INSTRUCTION_DEF( 0x30, 1, 1, "faload", NULL ),
	INSTRUCTION_DEF( 0x31, 1, 1, "daload", NULL ),
	INSTRUCTION_DEF( 0x32, 1, 1, "aaload", NULL ),
	INSTRUCTION_DEF( 0x33, 1, 1, "baload", NULL ),
	INSTRUCTION_DEF( 0x34, 1, 1, "caload", NULL ),
	INSTRUCTION_DEF( 0x35, 1, 1, "saload", NULL ),
	INSTRUCTION_DEF( 0x36, 2, 2, "istore", "B" ),
	INSTRUCTION_DEF( 0x37, 2, 2, "lstore", "B" ),
	INSTRUCTION_DEF( 0x38, 2, 2, "fstore", "B" ),
	INSTRUCTION_DEF( 0x39, 2, 2, "dstore", "B" ),
	INSTRUCTION_DEF( 0x3a, 2, 2, "astore", "B" ),
	INSTRUCTION_DEF( 0x3b, 1, 1, "istore_0", NULL ),
	INSTRUCTION_DEF( 0x3c, 1, 1, "istore_1", NULL ),
	INSTRUCTION_DEF( 0x3d, 1, 1, "istore_2", NULL ),
	INSTRUCTION_DEF( 0x3e, 1, 1, "istore_3", NULL ),
	INSTRUCTION_DEF( 0x3f, 1, 1, "lstore_0", NULL ),
	INSTRUCTION_DEF( 0x40, 1, 1, "lstore_1", NULL ),
	INSTRUCTION_DEF( 0x41, 1, 1, "lstore_2", NULL ),
	INSTRUCTION_DEF( 0x42, 1, 1, "lstore_3", NULL ),
	INSTRUCTION_DEF( 0x43, 1, 1, "fstore_0", NULL ),
	INSTRUCTION_DEF( 0x44, 1, 1, "fstore_1", NULL ),
	INSTRUCTION_DEF( 0x45, 1, 1, "fstore_2", NULL ),
	INSTRUCTION_DEF( 0x46, 1, 1, "fstore_3", NULL ),
	INSTRUCTION_DEF( 0x47, 1, 1, "dstore_0", NULL ),
	INSTRUCTION_DEF( 0x48, 1, 1, "dstore_1", NULL ),
	INSTRUCTION_DEF( 0x49, 1, 1, "dstore_2", NULL ),
	INSTRUCTION_DEF( 0x4a, 1, 1, "dstore_3", NULL ),
	INSTRUCTION_DEF( 0x4b, 1, 1, "astore_0", NULL ),
	INSTRUCTION_DEF( 0x4c, 1, 1, "astore_1", NULL ),
	INSTRUCTION_DEF( 0x4d, 1, 1, "astore_2", NULL ),
	INSTRUCTION_DEF( 0x4e, 1, 1, "astore_3", NULL ),
	INSTRUCTION_DEF( 0x4f, 1, 1, "iastore", NULL ),
	INSTRUCTION_DEF( 0x50, 1, 1, "lastore", NULL ),
	INSTRUCTION_DEF( 0x51, 1, 1, "fastore", NULL ),
	INSTRUCTION_DEF( 0x52, 1, 1, "dastore", NULL ),
	INSTRUCTION_DEF( 0x53, 1, 1, "aastore", NULL ),
	INSTRUCTION_DEF( 0x54, 1, 1, "bastore", NULL ),
	INSTRUCTION_DEF( 0x55, 1, 1, "castore", NULL ),
	INSTRUCTION_DEF( 0x56, 1, 1, "sastore", NULL ),
	INSTRUCTION_DEF( 0x57, 1, 1, "pop", NULL ),
	INSTRUCTION_DEF( 0x58, 1, 1, "pop2", NULL ),
	INSTRUCTION_DEF( 0x59, 1, 1, "dup", NULL ),
	INSTRUCTION_DEF( 0x5a, 1, 1, "dup_x1", NULL ),
	INSTRUCTION_DEF( 0x5b, 1, 1, "dup_x2", NULL ),
	INSTRUCTION_DEF( 0x5c, 1, 1, "dup2", NULL ),
	INSTRUCTION_DEF( 0x5d, 1, 1, "dup2_x1", NULL ),
	INSTRUCTION_DEF( 0x5e, 1, 1, "dup2_x2", NULL ),
	INSTRUCTION_DEF( 0x5f, 1, 1, "swap", NULL ),
	INSTRUCTION_DEF( 0x60, 1, 1, "iadd", NULL ),
	INSTRUCTION_DEF( 0x61, 1, 1, "ladd", NULL ),
	INSTRUCTION_DEF( 0x62, 1, 1, "fadd", NULL ),
	INSTRUCTION_DEF( 0x63, 1, 1, "dadd", NULL ),
	INSTRUCTION_DEF( 0x64, 1, 1, "isub", NULL ),
	INSTRUCTION_DEF( 0x65, 1, 1, "lsub", NULL ),
	INSTRUCTION_DEF( 0x66, 1, 1, "fsub", NULL ),
	INSTRUCTION_DEF( 0x67, 1, 1, "dsub", NULL ),
	INSTRUCTION_DEF( 0x68, 1, 1, "imul", NULL ),
	INSTRUCTION_DEF( 0x69, 1, 1, "lmul", NULL ),
	INSTRUCTION_DEF( 0x6a, 1, 1, "fmul", NULL ),
	INSTRUCTION_DEF( 0x6b, 1, 1, "dmul", NULL ),
	INSTRUCTION_DEF( 0x6c, 1, 1, "idiv", NULL ),
	INSTRUCTION_DEF( 0x6d, 1, 1, "ldiv", NULL ),
	INSTRUCTION_DEF( 0x6e, 1, 1, "fdiv", NULL ),
	INSTRUCTION_DEF( 0x6f, 1, 1, "ddiv", NULL ),
	INSTRUCTION_DEF( 0x70, 1, 1, "irem", NULL ),
	INSTRUCTION_DEF( 0x71, 1, 1, "lrem", NULL ),
	INSTRUCTION_DEF( 0x72, 1, 1, "frem", NULL ),
	INSTRUCTION_DEF( 0x73, 1, 1, "drem", NULL ),
	INSTRUCTION_DEF( 0x74, 1, 1, "ineg", NULL ),
	INSTRUCTION_DEF( 0x75, 1, 1, "lneg", NULL ),
	INSTRUCTION_DEF( 0x76, 1, 1, "fneg", NULL ),
	INSTRUCTION_DEF( 0x77, 1, 1, "dneg", NULL ),
	INSTRUCTION_DEF( 0x78, 1, 1, "ishl", NULL ),
	INSTRUCTION_DEF( 0x79, 1, 1, "lshl", NULL ),
	INSTRUCTION_DEF( 0x7a, 1, 1, "ishr", NULL ),
	INSTRUCTION_DEF( 0x7b, 1, 1, "lshr", NULL ),
	INSTRUCTION_DEF( 0x7c, 1, 1, "iushr", NULL ),
	INSTRUCTION_DEF( 0x7d, 1, 1, "lushr", NULL ),
	INSTRUCTION_DEF( 0x7e, 1, 1, "iand", NULL ),
	INSTRUCTION_DEF( 0x7f, 1, 1, "land", NULL ),
	INSTRUCTION_DEF( 0x80, 1, 1, "ior", NULL ),
	INSTRUCTION_DEF( 0x81, 1, 1, "lor", NULL ),
	INSTRUCTION_DEF( 0x82, 1, 1, "ixor", NULL ),
	INSTRUCTION_DEF( 0x83, 1, 1, "lxor", NULL ),
	INSTRUCTION_DEF( 0x84, 3, 3, "iinc", "BB" ),
	INSTRUCTION_DEF( 0x85, 1, 1, "i2l", NULL ),
	INSTRUCTION_DEF( 0x86, 1, 1, "i2f", NULL ),
	INSTRUCTION_DEF( 0x87, 1, 1, "i2d", NULL ),
	INSTRUCTION_DEF( 0x88, 1, 1, "l2i", NULL ),
	INSTRUCTION_DEF( 0x89, 1, 1, "l2f", NULL ),
	INSTRUCTION_DEF( 0x8a, 1, 1, "l2d", NULL ),
	INSTRUCTION_DEF( 0x8b, 1, 1, "f2i", NULL ),
	INSTRUCTION_DEF( 0x8c, 1, 1, "f2l", NULL ),
	INSTRUCTION_DEF( 0x8d, 1, 1, "f2d", NULL ),
	INSTRUCTION_DEF( 0x8e, 1, 1, "d2i", NULL ),
	INSTRUCTION_DEF( 0x8f, 1, 1, "d2l", NULL ),
	INSTRUCTION_DEF( 0x90, 1, 1, "d2f", NULL ),
	INSTRUCTION_DEF( 0x91, 1, 1, "i2b", NULL ),
	INSTRUCTION_DEF( 0x92, 1, 1, "i2c", NULL ),
	INSTRUCTION_DEF( 0x93, 1, 1, "i2s", NULL ),
	INSTRUCTION_DEF( 0x94, 1, 1, "lcmp", NULL ),
	INSTRUCTION_DEF( 0x95, 1, 1, "fcmpl", NULL ),
	INSTRUCTION_DEF( 0x96, 1, 1, "fcmpg", NULL ),
	INSTRUCTION_DEF( 0x97, 1, 1, "dcmpl", NULL ),
	INSTRUCTION_DEF( 0x98, 1, 1, "dcmpg", NULL ),
	INSTRUCTION_DEF( 0x99, 3, 0, "ifeq",      "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0x9a, 3, 0, "ifne",      "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0x9b, 3, 0, "iflt",      "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0x9c, 3, 0, "ifge",      "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0x9d, 3, 0, "ifgt",      "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0x9e, 3, 0, "ifle",      "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0x9f, 3, 0, "if_icmpeq", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa0, 3, 0, "if_icmpne", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa1, 3, 0, "if_icmplt", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa2, 3, 0, "if_icmpge", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa3, 3, 0, "if_icmpgt", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa4, 3, 0, "if_icmple", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa5, 3, 0, "if_acmpeq", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa6, 3, 0, "if_acmpne", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xa7, 3, 0, "goto",      "S" ), /* incr always n */
	INSTRUCTION_DEF( 0xa8, 3, 0, "jsr",       "S" ), /* incr always n */
	INSTRUCTION_DEF( 0xa9, 2, 0, "ret",       "B" ), /* incr always v */
	INSTRUCTION_DEF( 0xaa, 0, 0, "tableswitch",  NULL ), /* size varies */
	INSTRUCTION_DEF( 0xab, 0, 0, "lookupswitch", NULL ), /* size varies */
	INSTRUCTION_DEF( 0xac, 1, 0, "ireturn", NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xad, 1, 0, "lreturn", NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xae, 1, 0, "freturn", NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xaf, 1, 0, "dreturn", NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xb0, 1, 0, "areturn", NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xb1, 1, 0, "return",  NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xb2, 3, 3, "getstatic", "f" ),
	INSTRUCTION_DEF( 0xb3, 3, 3, "putstatic", "f" ),
	INSTRUCTION_DEF( 0xb4, 3, 3, "getfield",  "f" ),
	INSTRUCTION_DEF( 0xb5, 3, 3, "putfield",  "f" ),
	INSTRUCTION_DEF( 0xb6, 3, 3, "invokevirtual",   "m" ),  /* NB: must incr to next */
	INSTRUCTION_DEF( 0xb7, 3, 3, "invokespecial",   "m" ),  /* NB: must incr to next */
	INSTRUCTION_DEF( 0xb8, 3, 3, "invokestatic",    "m" ),  /* NB: must incr to next */
	INSTRUCTION_DEF( 0xb9, 5, 5, "invokeinterface", "vB" ), /* NB: must incr to next and skip zero byte */
	INSTRUCTION_DEF( 0xba, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xbb, 3, 3, "new",        "c" ),
	INSTRUCTION_DEF( 0xbc, 2, 2, "newarray",   "t" ),
	INSTRUCTION_DEF( 0xbd, 3, 3, "anewarray",  "c" ),
	INSTRUCTION_DEF( 0xbe, 1, 1, "arraylength", NULL ),
	INSTRUCTION_DEF( 0xbf, 1, 0, "athrow",      NULL ), /* incr always 0 */
	INSTRUCTION_DEF( 0xc0, 3, 3, "checkcast",  "c" ),
	INSTRUCTION_DEF( 0xc1, 3, 3, "instanceof", "c" ),
	INSTRUCTION_DEF( 0xc2, 1, 1, "monitorenter", NULL ),
	INSTRUCTION_DEF( 0xc3, 1, 1, "monitorexit",  NULL ),
	INSTRUCTION_DEF( 0xc4, 1, 0, "wide", NULL ),
	INSTRUCTION_DEF( 0xc5, 4, 4, "multianewarray", "cB" ),
	INSTRUCTION_DEF( 0xc6, 3, 0, "ifnull",    "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xc7, 3, 0, "ifnonnull", "S" ), /* incr size or n */
	INSTRUCTION_DEF( 0xc8, 5, 0, "goto_w",    "I" ), /* incr always n */
	INSTRUCTION_DEF( 0xc9, 5, 0, "jsr_w",     "I" ), /* incr always n */
	INSTRUCTION_DEF( 0xca, 0, 0, NULL, NULL ),   /* breakpoint */
	INSTRUCTION_DEF( 0xcb, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xcc, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xcd, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xce, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xcf, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd0, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd1, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd2, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd3, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd4, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd5, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd6, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd7, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd8, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xd9, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xda, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xdb, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xdc, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xdd, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xde, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xdf, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe0, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe1, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe2, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe3, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe4, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe5, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe6, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe7, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe8, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xe9, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xea, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xeb, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xec, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xed, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xee, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xef, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf0, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf1, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf2, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf3, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf4, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf5, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf6, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf7, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf8, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xf9, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xfa, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xfb, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xfc, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xfd, 0, 0, NULL, NULL ),   /* UNDEFINED */
	INSTRUCTION_DEF( 0xfe, 0, 0, NULL, NULL ),   /* impdep1 */
	INSTRUCTION_DEF( 0xff, 0, 0, NULL, NULL ),   /* impdep2 */
};
