/******************************************************************************
 *
 * Filename: jstring.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  16th April 2003
 *
 * Purpose:  Java-style string routines for convenience.
 *
 * (c) Copyright R.H.Thornburrow, 1998-2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/


#include <string.h>

#include "config.h"

#ifdef WIN32
#include "memory.h"
#endif /* WIN32 */

#include "jstring.h"


/**
 * Eqivalent of Java's String.indexOf( int, int ).
 */
int indexOf( const char* str, char c, int start )
{
	int n;
	for( n = start; str[n] && (str[n] != c); n++ );
	return str[n] ? n : -1;
}


/**
 * Java-style hash function.  Calculates a simple hash code for a given string.
 * For long strings, we will actully overflow, but that does not matter.
 */
u4 hash( const char* str )
{
	int32 n;
	u4 ret = 0, off = 0;
	int len = strlen( str );

	if( len < 16 )
	{
 	    for( n = len; n > 0; n-- )
	 		ret = (ret * 37) + str[off++];
 	}
	else
	{
 	    int skip = len / 8;
 	    for( n = len ; n > 0; n -= skip, off += skip )
 			ret = (ret * 39) + str[off];
 	}

	return ret;
}
