/******************************************************************************
 * 
 * Filename: math-marm.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  4th December 1999
 *
 * Purpose:  Missing maths routines in WINS libraries.  These routines are not
 *			 called directly, but are a results of maths operations and casts.
 *
 * (c) Copyright R.H.Thornburrow, 1999-2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "config.h"
#include "types.h"

#ifndef PLATFORM_EPOC32_MARM
#error "This file is only intended for MARM (Epoc32/Psion Series 5)"
#endif /* PLATFORM_EPOC32_WINS */


/* double to int64 conversion */
int64 __fixdfdi( double d )
{
    return (long) d;
}

/* float to int64 conversion */
int64 __fixsfdi( float f )
{
    return (long) f;
}

int32 __fixunssfsi( float f )
{
    return (int) f;
}

/* int64 to double conversion */
double __floatdidf( int64 l )
{
    return (double) (long) l;
}

/* int64 to float conversion */
float __floatdisf( int64 l )
{
    return (float) (long) l;
}

int64 __muldi3( int64 a, int64 b )
{
    return (long) a * (long) b;
}

int64 __ashldi3( int64 a, int64 b )
{
    return (long) a << (long) b;
}

int64 __ashrdi3( int64 a, int64 b )
{
    return (long) a >> (long) b;
}

int64 __divdi3( int64 a, int64 b )
{
    return (long) a / (long) b;
}

int64 __moddi3( int64 a, int64 b )
{
    return (long) a % (long) b;
}

/* int64 shift-left */
int64 _allshl( int64 a, int64 b )
{
    return (long) a << (long) b;
}

/* int64 shift-right */
int64 _allshr( int64 a, int64 b )
{
    return (long) a >> (long) b;
}
