/******************************************************************************
 *
 * Filename: java_io_FileDescriptor.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Native method implementations for java.io.FileDescriptor.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "../engine/config.h"

#include "java_io_FileDescriptor.h"

#include "../engine/openfiles.h"
#include "../engine/engine.h"


OpenFile_Entry in  = { oft_BuiltIn, NULL, 0 };
OpenFile_Entry out = { oft_BuiltIn, NULL, 0 };
OpenFile_Entry err = { oft_BuiltIn, NULL, 0 };


#if JDK_VERSION < 140
/* static java.io.FileDescriptor initSystemFD(java.io.FileDescriptor fdObj, int desc); */
jobject Java_java_io_FileDescriptor_initSystemFD( JNIEnv* env, jclass staticself, jobject fdObj, jint desc )
{
	switch( desc )
	{
	case 0: in.f  = stdin;  openfilesv[0] = &in; break;
	case 1: out.f = stdout; openfilesv[1] = &out; break;
	case 2: err.f = stderr; openfilesv[2] = &err; break;
	}

	fdObj->data[0] = desc;

	return fdObj;
}
#endif /* JDK_VERSION 1.4.0 */


#if JDK_VERSION >= 140
/**
 * private static native void initIDs();
 */
void Java_java_io_FileDescriptor_initIDs( JNIEnv* env, jclass staticself )
{
	/* If we haven't already been initialized, do so */
	if( !openfilesv[0] )
	{
		in.f  = stdin;  openfilesv[0] = &in;
		out.f = stdout; openfilesv[1] = &out;
		err.f = stderr; openfilesv[2] = &err;
	}
}
#endif /* JDK_VERSION 1.4.0 */
