/******************************************************************************
 *
 * Filename: `.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  29th March 2002
 *
 * Purpose:  Native method implementations for java.io.ObjectInputStream.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "../engine/config.h"
#include "../engine/util.h"
#include "../engine/globals.h"

#include "java_io_ObjectInputStream.h"


/* Resolve a class name relative to the specified class.  If the
 * class is null find the first available class loader up the
 * stack.  This will resolve classes relative to the caller of
 * ObjectInputStream instead of the itself. Classes must be
 * loaded/resolved relative to the application.
 *
 * private native Class loadClass0(Class cl, String classname)
 *     throws ClassNotFoundException;
 */
jclass Java_java_io_ObjectInputStream_loadClass0( JNIEnv* env, jobject self, jclass cl, jobject str )
{
	char classname[255];

	if( extractFromJavaLangString( classname, 255, str ) )
	{
		ClassFile* cf;
		u4 n;

		for( n = 0; n < strlen( classname ); n++ )
		{
			if( classname[n] == '.' )
				classname[n] = '/';
		}

		/* ClassFile pointer is last field of cl */
		printf( "WARNING: ObjectInputStream.loadClass0( \"%s\", \"%s\" ) dummied.\n",
			(cl ? ((ClassFile*) cl->data[cl->cf->object_size])->name : "null"), classname );

		if( !( cf = getClass( env, classname ) ) )
		{
			if( verbose )
				printf( "Class.forName() failed to load class \"%s\"\n", classname );

			/* all exceptions translate into ClassNotFoundException's */
			env->retcode = newCompleteException( env, ClassNotFoundException, classname );
			return NULL;
		}

		return cf->classobj;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return NULL;
}
