/******************************************************************************
 *
 * Filename: java_lang_StrictMath.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  7th May 2003
 *
 * Purpose:  Native method implementations for java.lang.StrictMath.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <math.h>

#include "../engine/config.h"

#include "java_lang_StrictMath.h"


/* JDK 1.4 (class doesn't exist in 1.1.7A)
 1.4 public static native strictfp double sin(double);
 1.4 public static native strictfp double cos(double);
 1.4 public static native strictfp double tan(double);
 1.4 public static native strictfp double asin(double);
 1.4 public static native strictfp double acos(double);
 1.4 public static native strictfp double atan(double);
 1.4 public static native strictfp double exp(double);
+1.4 public static native strictfp double log(double);
 1.4 public static native strictfp double sqrt(double);
 1.4 public static native strictfp double IEEEremainder(double, double);
 1.4 public static native strictfp double ceil(double);
+1.4 public static native strictfp double floor(double);
 1.4 public static native strictfp double rint(double);
 1.4 public static native strictfp double atan2(double, double);
 1.4 public static native strictfp double pow(double, double);
*/

#if JDK_VERSION >= 140

/**
 * Returns the largest integer (expressed as a double) that is not greater than
 * 'a'.
 *
 * @since  JDK1.4 (at least JDK1.2)
 *
 * public static native strictfp double floor( double a );
 */
jdouble Java_java_lang_StrictMath_floor( JNIEnv* env, jclass staticself, jdouble a )
{
	return floor( a );
}


/**
 * Returns the natural logarithm (ln or log to the base e) of 'a'. 'a' must be
 * a number greater than 0.0.
 *
 * @since   JDK1.4 (at least JDK1.2)
 *
 * public static native strictfp double log( double a );
 */
jdouble Java_java_lang_StrictMath_log( JNIEnv* env, jclass staticself, jdouble a )
{
	return log( a );
}

#endif /* JDK 1.4 */
