/******************************************************************************
 *
 * Filename: java_lang_String.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  21st April 2003
 *
 * Purpose:  Native method implementations for java.lang.String.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "../engine/config.h"

#include "java_lang_String.h"

#include "../engine/util.h"
#include "../engine/hashtable.h"


Hashtable* internStrings = NULL;


/**
 * Returns a canonical representation for the string object. 
 *
 * If s and t are strings such that s.equals(t), it is guaranteed that
 * s.intern() == t.intern().
 *
 * @return  a string that has the same contents as this string, but is
 *          guaranteed to be from a pool of unique strings.
 *
 * public native java.lang.String intern();
 */
jobject Java_java_lang_String_intern( JNIEnv* env, jobject self )
{
	jobject existing;
	char aString[255];

	if( extractFromJavaLangString( aString, 255, self ) )
	{
		if( !internStrings )
			internStrings = Hashtable_new( 32 );

		if( (existing = Hashtable_get( internStrings, aString )) != NULL )
		{
			return existing;
		}

		Hashtable_put( internStrings, strdup(aString), self );
		return self;
	}

	printf( "ERROR: java.lang.String.intern() unable to extract java.lang.String.\n" );
	return self;
}
