/******************************************************************************
 *
 * Filename: java_lang_Thread.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  25th March 2002
 *
 * Purpose:  Native method implementations for java.lang.Thread.
 *
 * (c) Copyright R.H.Thornburrow, 2002-3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "../engine/config.h"
#include "../engine/engine.h"
#include "../engine/globals.h"
#include "../engine/util.h"

#include "java_lang_Thread.h"


/* JDK 1.1.7A & 1.4.0
1.4 private static native void registerNatives();
+   public static native java.lang.Thread currentThread();
    public static native void yield();
    public static native void sleep(long) throws java.lang.InterruptedException;
+   public synchronized native void start();
    private native boolean isInterrupted(boolean);
    public final native boolean isAlive();
    public native int countStackFrames();
1.4 public static native boolean holdsLock(java.lang.Object);
+   private native void setPriority0(int);
    private native void stop0(java.lang.Object);
    private native void suspend0();
    private native void resume0();
    private native void interrupt0();
*/


#if JDK_VERSION >= 140
/**
 * private static native void registerNatives();
 */
void Java_java_lang_Thread_registerNatives( JNIEnv* env, jclass staticself )
{
	/* Do Nothing! */
}
#endif /* JDK_VERSION 1.4.0 */


/**
 * Returns a reference to the current thread.
 *
 * @since   JDK1.0
 *
 * public static native Thread currentThread();
 */
jobject Java_java_lang_Thread_currentThread( JNIEnv* env, jclass staticself )
{
	return threads[0];
}


/**
 * Returns true if this thread if has been started and hasn't died. 
 *
 * @since   JDK1.0
 *
 * public final native boolean isAlive();
 */
jboolean Java_java_lang_Thread_isAlive( JNIEnv* env, jobject self )
{
	if( self == threads[0] )
		return JNI_TRUE;

	return JNI_FALSE;
}


/**
 * private native void setPriority0(int newPriority);
 */
void Java_java_lang_Thread_setPriority0( JNIEnv* env, jclass staticself, jint newPriority )
{
	printf( "WARNING: Thread.setPriority( %d ) dummied.\n", (int) newPriority );
}


/**
 * Causes this thread to begin execution.  The VM actually spawns a new thread,
 * which will begin execution at the run() method of this Thread object.  The
 * current Thread gets control back immediately.
 *
 * Currently throws an InternalError as we don't yet support multiple threads.
 *
 * @exception  IllegalThreadStateException  if the thread is already running.
 * @since      JDK1.0
 *
 * public synchronized native void start();
 */
void Java_java_lang_Thread_start( JNIEnv* env, jobject self )
{
#if JDK_VERSION < 140
	env->retcode = newCompleteException( env, InternalError, "Threads not supported." );
#else
	char buf[256];
	ObjRef* name = (ObjRef*) ((javaLangThread*) self->data)->name;
	w2cstrncpy( buf, (u2*) name->data, name->length );
	buf[name->length] = '\0';
	printf( "WARNING: Thread.start() for thread \"%s\" dummied.\n", buf );
#endif /* JDK 1.4 */
}
