/******************************************************************************
 *
 * Filename: java_lang_reflect_Field.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  17th July 2003
 *
 * Purpose:  Native method implementations for java.lang.reflect.Field.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "../engine/config.h"

#include "java_lang_reflect_Field.h"

#include "../engine/engine.h"
#include "../engine/util.h"
#include "../engine/disasm.h"


/* JDK 1.1.7A
+#  public native int getModifiers();
 #  public native java.lang.Object get(java.lang.Object);
 #  public native boolean getBoolean(java.lang.Object);
 #  public native byte getByte(java.lang.Object);
 #  public native char getChar(java.lang.Object);
 #  public native short getShort(java.lang.Object);
 #  public native int getInt(java.lang.Object);
 #  public native long getLong(java.lang.Object);
 #  public native float getFloat(java.lang.Object);
 #  public native double getDouble(java.lang.Object);
 #  public native void set(java.lang.Object, java.lang.Object);
 #  public native void setBoolean(java.lang.Object, boolean);
 #  public native void setByte(java.lang.Object, byte);
 #  public native void setChar(java.lang.Object, char);
 #  public native void setShort(java.lang.Object, short);
 #  public native void setInt(java.lang.Object, int);
 #  public native void setLong(java.lang.Object, long);
 #  public native void setFloat(java.lang.Object, float);
 #  public native void setDouble(java.lang.Object, double);
*/

/* Fields are:
    java.lang.Class		clazz;
    int					slot;
    java.lang.String	name;
    java.lang.Class		type;
*/


#if JDK_VERSION < 140
/**
 * Returns the access flags for the field this class represents.
 *
 * @since JDK1.1? (not in JDK 1.4)
 *
 * public native int getModifiers();
 */
jint Java_java_lang_reflect_Field_getModifiers( JNIEnv* env, jobject self )
{
	int32 clazzOffset, nameOffset;
	jclass clazz;
	ClassFile *cf;
	u2 index;
	char name[255];
	int found = 0;
	
	clazzOffset = getFieldOffset( self->cf, "clazz", "Ljava/lang/Class;" );
	nameOffset  = getFieldOffset( self->cf, "name", "Ljava/lang/String;" );

	if( (clazzOffset < 0) || (nameOffset < 0) )
	{
		env->retcode = newCompleteException( env, InternalError, "java.lang.reflect.Field does not contain correct fields" );
		return 0;
	}

	if( !extractFromJavaLangString( name, 255, (ObjRef*) self->data[nameOffset] ) )
	{
		env->retcode = newCompleteException( env, InternalError, "Unable to extract field name" );
		return 0;
	}

	clazz = (jclass) self->data[clazzOffset];

	cf = (ClassFile*) clazz->data[clazz->cf->object_size];    /* last field is cf */

	/* For each field check its name to see if it matched the one we are looking for */
	for( index = 0; index < cf->fields_count; index++ )
	{
		/* Is this the field we want? */
		if( checkSimpleName( cf, cf->fields[index]->name_index, name ) )
		{
			found = 1;
			break;
		}
	}

	/* Did we find anything? If not, raise a NoSuchFieldException */
	if( index >= cf->fields_count )
	{
		env->retcode = newCompleteException( env, NoSuchFieldException, name );
		return 0;
	}

	return cf->fields[index]->access_flags;
}
#endif /* JDK 1.4 */
