/******************************************************************************
 *
 * Filename: java_security_AccessController.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  12th April 2003
 *
 * Purpose:  Native method implementations for java.reflect.Reflection.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "../engine/config.h"

#include "sun_reflect_Reflection.h"

#include "../engine/engine.h"


/* JDK 1.4
    public static native java.lang.Class getCallerClass(int);
    private static native int getClassAccessFlags(java.lang.Class);
*/


#if JDK_VERSION >= 140
/**
 * Returns the calling class 'levels' stack frames back up the call stack.
 * Note that the stack frame for this method needs to be taken into
 * consideration, therefore the class at level 1 refers to the caller of this
 * method.
 *
 * private static native Class getCallerClass( int levels );
 */
jobject Java_sun_reflect_Reflection_getCallerClass( JNIEnv* env, jclass staticself, jint levels )
{
	Frame* next;
	int n;

	/* count back 'levels' stack frames */
	for( n = 0, next = env->fp; next && (n < levels - 1); next = next->next )
		n++;

	if( next && next->cf )
	{
		/*printf( "sun.reflect.Reflection.getCallerClass( %d ) returning %s\n", levels, next->cf->name );*/
		return next->cf->classobj;
	}

	printf( "WARNING: sun.reflect.Reflection.getCallerClass( %d ) - not enough stack frames.\n", levels );
	return NULL;
}
#endif /* JDK_VERSION 1.4.0 */


#if JDK_VERSION >= 140
/**
 * Returns the access flags field for the given class (part of classfile).
 *
 * private static native int getClassAccessFlags(java.lang.Class);
 */
jint Java_sun_reflect_Reflection_getClassAccessFlags( JNIEnv* env, jclass staticself, jclass aClass )
{
	ClassFile* cf = (ClassFile*) aClass->data[aClass->cf->object_size];    /* last field is cf */
	return cf->access_flags;
}
#endif /* JDK 1.4 */
